/******************************************************************************
 * \file QSPI1.c
 * \copyright Copyright (C) Infineon Technologies AG 2019
 *
 * Use of this file is subject to the terms of use agreed between (i) you or
 * the company in which ordinary course of business you are acting and (ii)
 * Infineon Technologies AG or its licensees. If and as long as no such terms
 * of use are agreed, use of this file is subject to following:
 *
 * Boost Software License - Version 1.0 - August 17th, 2003
 *
 * Permission is hereby granted, free of charge, to any person or organization
 * obtaining a copy of the software and accompanying documentation covered by
 * this license (the "Software") to use, reproduce, display, distribute,
 * execute, and transmit the Software, and to prepare derivative works of the
 * Software, and to permit third-parties to whom the Software is furnished to
 * do so, all subject to the following:
 *
 * The copyright notices in the Software and this entire statement, including
 * the above license grant, this restriction and the following disclaimer, must
 * be included in all copies of the Software, in whole or in part, and all
 * derivative works of the Software, unless such copies or derivative works are
 * solely in the form of machine-executable object code generated by a source
 * language processor.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE, TITLE AND NON-INFRINGEMENT. IN NO EVENT
 * SHALL THE COPYRIGHT HOLDERS OR ANYONE DISTRIBUTING THE SOFTWARE BE LIABLE
 * FOR ANY DAMAGES OR OTHER LIABILITY, WHETHER IN CONTRACT, TORT OR OTHERWISE,
 * ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
 * DEALINGS IN THE SOFTWARE.
 *****************************************************************************/
#include "Ifx_reg.h"
#include "ifxScuWdt.h"
#include "intpriodef.h"
#include "ports.h"
#include "qspi_private.h"
#include "dma_private.h"

#include "qspi1_public.h"
volatile uint32 __attribute__((section (".data.data_cpu0"))) dmaCh3Cnt;
volatile uint32 __attribute__((section (".data.data_cpu0"))) dmaCh4Cnt;
volatile uint32 __attribute__((section (".data.data_cpu0"))) QSPI1_errorCount;

/* Defines the basic configuration parameters for the current slave select. It can be read by
 * software, or written through a write to the TXFIFO.
 */
Ifx_QSPI_BACON __attribute__((section (".data.data_cpu0"))) baconEntrySpi1Start =
{
  /* Slave Channel Select for timing */
  .B.CS = 9,
  /* Data Length - 1 */
#if QSPI_DL==DMA_CH_SIZE_INT8
  .B.DL = 7,
#elif QSPI_DL==DMA_CH_SIZE_INT16
  .B.DL = 15,
#else
  .B.DL = 31,
#endif
  /* length in bits or bytes */
  .B.BYTE = qspi_baconEntry_byte_Bits,
  /* Shift Mode */
  .B.MSB = qspi_baconEntry_Shift_MSB_first,
  /* User Interrupt at the PT1 Event in the Subsequent Frames */
  .B.UINT = qspi_baconEntry_UserInterruptPT1_EventDisabled,
  /* Parity Type */
  .B.PARTYP = qspi_baconEntry_partyp_Even,
  .B.TRAIL = 0,  /* Trailing Delay Length*/
  .B.TPRE  = 0,  /* Prescaler for the Trailing Delay */
  .B.LEAD  = 2,  /* Leading Delay Length */
  .B.LPRE  = 2,  /* Prescaler for the Leading Delay */
  .B.IDLE  = 2,  /* Idle Delay Length */
  .B.IPRE  = 2,  /* Prescaler for the Idle Delay */
  /* Last Word in a Frame */
  .B.LAST  = qspi_baconEntry_last_false,
};

Ifx_QSPI_BACON __attribute__((section (".data.data_cpu0"))) baconEntrySpi1Stop =
{
  /* Slave Channel Select for timing */
  .B.CS = 9,
  /* Data Length - 1 */
#if QSPI_DL==DMA_CH_SIZE_INT8
  .B.DL = 7,
#elif QSPI_DL==DMA_CH_SIZE_INT16
  .B.DL = 15,
#else
  .B.DL = 31,
#endif
  /* length in bits or bytes */
  .B.BYTE = qspi_baconEntry_byte_Bits,
  /* Shift Mode */
  .B.MSB = qspi_baconEntry_Shift_MSB_first,
  /* User Interrupt at the PT1 Event in the Subsequent Frames */
  .B.UINT = qspi_baconEntry_UserInterruptPT1_EventDisabled,
  /* Parity Type */
  .B.PARTYP = qspi_baconEntry_partyp_Even,
  .B.TRAIL = 2,  /* Trailing Delay Length*/
  .B.TPRE  = 2,  /* Prescaler for the Trailing Delay */
  .B.LEAD  = 0,  /* Leading Delay Length */
  .B.LPRE  = 0,  /* Prescaler for the Leading Delay */
  .B.IDLE  = 2,  /* Idle Delay Length */
  .B.IPRE  = 2,  /* Prescaler for the Idle Delay */
  /* Last Word in a Frame */
  .B.LAST  = qspi_baconEntry_last_true,
};

/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
void QSPI1_Init (void)
{
  uint16 cpuWdtPassword = IfxScuWdt_getCpuWatchdogPassword();

  IfxScuWdt_clearCpuEndinit(cpuWdtPassword);
  QSPI1_CLC.U = 0x0; /* enable peripheral and allow sleep mode */
  (void) QSPI1_CLC.U; /* read the register to force the write out of the cpu */

  /* Kernel reset */
  QSPI1_KRST0.U = 0x01;
  QSPI1_KRST1.U = 0x01;
  IfxScuWdt_setCpuEndinit(cpuWdtPassword);

  /* wait for the kernel to be reset before proceeding */
  while ((QSPI1_KRST0.U & 0x02) != 0x02)
    ;

  /* Clear Reset done flag */
  IfxScuWdt_clearCpuEndinit(cpuWdtPassword);
  QSPI1_KRSTCLR.U = 0x01;
  IfxScuWdt_setCpuEndinit(cpuWdtPassword);

  /* GLOBALCON contains configuration parameters which affect all channels */
  const Ifx_QSPI_GLOBALCON globalcon = {
    .B.TQ = 0, /* Global DIVIDER VALUE IS (TQ+1) */
    .B.SI = qspi_globalcon_si_Disabled, /* Status Injection */
    .B.EXPECT = qspi_globalcon_expect_1024, /* Time-Out Value for the Expect Phase */
    .B.LB = qspi_globalcon_lb_Inactive, /* Loop-Back Control */
    .B.DEL0 = qspi_globalcon_del0_Off, /* Delayed Mode for SLSO0 */
    .B.STROBE = qspi_globalcon_strobe_NotUsed, /* Strobe Delay for SLSO0 in Delayed Mode */
    .B.SRF = qspi_globalcon_srf_disabled, /* Stop on RxFIFO Full */
    .B.STIP = qspi_globalcon_stip_NotUsed, /* Slave Transmit Idle State Polarity */
    .B.EN = qspi_globalcon_en_PAUSE, /* Enable Bit */
    .B.MS = qspi_globalcon_ms_MasterTransmitReceive, /* Master or Slave Mode */
    .B.AREN = qspi_globalcon_aren_AutomaticResetEnabled, /* Automatic Reset Enable */
    .B.CLKSEL = qspi_globalcon_PeripheralClock,
    .B.RESETS = qspi_globalcon_resets_default /* Bits for resetting sub-modules per software */
  };
  QSPI1_GLOBALCON.U = globalcon.U;

  /* The register QSPIx_BACON is not configured in the initialization routine, this is done by the user code */

  /* GLOBALCON1 contains bit fields for control of the extension of the slave select signals by using an external demultiplexer */
  const Ifx_QSPI_GLOBALCON1 globalcon1 = {
    .B.ERRORENS = qspi_globalcon1_errorens_AllErrorsEnabled, /* Error Enable Bits */
    .B.TXEN = qspi_globalcon1_txen_InterruptEnabled, /* TX Interrupt Event Enable */
    .B.RXEN = qspi_globalcon1_rxen_InterruptEnabled, /* RX Interrupt Event Enable */
    .B.PT1EN = qspi_globalcon1_pt1en_InterruptDisabled, /* Interrupt on PT1 Event Enable */
    .B.PT2EN = qspi_globalcon1_pt2en_InterruptDisabled, /* Interrupt on PT2 Event Enable */
    .B.USREN = qspi_globalcon1_usren_InterruptDisabled, /* Interrupt on USR Event Enable */
    .B.TXFIFOINT = qspi_globalcon1_txfifoint_Level2, /* Transmit FIFO Interrupt Threshold */
    .B.RXFIFOINT = qspi_globalcon1_rxfifoint_Level1, /* Receive FIFO Interrupt Threshold */
    .B.PT1 = qspi_globalcon1_pt1_EndOfFrame, /* Phase Transition Event 1 */
    .B.PT2 = qspi_globalcon1_pt2_EndOfWaitPhase, /* Phase Transition Event 2 */
    .B.TXFM = qspi_globalcon1_txfm_SingleMoveMode, /* TXFIFO Mode */
    .B.RXFM = qspi_globalcon1_rxfm_SingleMoveMode, /* RXFIFO Mode */
  };
  QSPI1_GLOBALCON1.U = globalcon1.U;

  /* configure slave select parameters */
  const Ifx_QSPI_ECON econ = {
    .B.Q = 24,   /* DIVIDER VALUE IS (Q+1) */
    .B.A = 1,  	/* DIVIDER VALUE IS (A+1) */
    .B.B = 0, 	/* Length expressed in time quantum of ECONz.Q */
    .B.C = 2, 	/* Length expressed in time quantum of ECONz.Q */
    .B.CPH = qspi_econ_cph_0, /* Clock Phase (mandatory setting) */
    .B.CPOL = qspi_econ_cpol_0, /* Clock Polarity (mandatory setting) */
    .B.PAREN = qspi_econ_paren_Disabled, /* Enable Parity Check */
#if QSPI_DL==DMA_CH_SIZE_INT8
    .B.BE = qspi_econ_be_SwapDisabled, /* Permutate bytes to / from Big Endian */
#elif QSPI_DL==DMA_CH_SIZE_INT16
    .B.BE = qspi_econ_be_Swap_16bit, /* Permutate bytes to / from Big Endian */
#else
    .B.BE = qspi_econ_be_Swap_32bit, /* Permutate bytes to / from Big Endian */
#endif

  };
  QSPI1_ECON1.U = econ.U; /*SLSO9*/

  /* select which slave select outputs */
  const Ifx_QSPI_SSOC ssoc = {
      .B.OEN = qspi_ssoc_oen_slso9,
      .B.AOL = qspi_ssoc_aol_none,
  };
  QSPI1_SSOC.U = ssoc.U;

  /*configure communication events */
  const Ifx_QSPI_STATUS1 status1 = {
    .B.SPD = qspi_status1_SpikeDetectionNoEvent, /* Spike Detection Flag */
    .B.SPDEN = qspi_status1_SpikeDetectionDisabled, /* Spike Detection Enable */
    .B.BRD = qspi_status1_BaudRateDeviationNoEvent, /* Baud Rate Deviation Flag */
    .B.BRDEN = qspi_status1_BaudRateDeviationDisabled, /* Baud Rate Deviation Enable */
  };
  QSPI1_STATUS1.U = status1.U;

#if 0 /* polling for Transmit and Receive */
  /* Enter interrupt level and initialize both the SRC and table entry */
  SRC_QSPI1RX.U  = TOS_CPU0 | SRE_OFF | SRPN_CPU0_QSPI1_RX;
  SRC_QSPI1TX.U  = TOS_CPU0 | SRE_OFF | SRPN_CPU0_QSPI1_TX;
#else /* use DMA */
  SRC_QSPI1TX.U  = TOS_DMA  | SRE_ON  | SRPN_DMA_CH_QSPI1_TX;
  SRC_QSPI1RX.U  = TOS_DMA  | SRE_ON  | SRPN_DMA_CH_QSPI1_RX;
#endif
  SRC_QSPI1U.U   = TOS_CPU0 | SRE_ON  | SRPN_CPU0_QSPI1_U;
  SRC_QSPI1PT.U  = TOS_CPU0 | SRE_ON  | SRPN_CPU0_QSPI1_PT;
  SRC_QSPI1ERR.U = TOS_CPU0 | SRE_ON  | SRPN_CPU0_QSPI1_ERR;

  /* select pins for the QSPI channel */
  const Ifx_QSPI_PISEL pisel = {
    .B.MRIS = qspi_pisel_mris_mrstxB,
    .B.SRIS = qspi_pisel_sris_mtsrxA,
    .B.SCIS = qspi_pisel_scis_sclkxA,
    .B.SLSIS = qspi_pisel_slsis_slsi_deselected,
  };
  QSPI1_PISEL.U = pisel.U;
  /*
   * P10.5 QSPI1 Slave Select Output 9 (ECON1)
   * P11.6 QSPI1 Master Clock Output
   * P11.3 QSPI1 Master Receive Input B
   * P11.9 QSPI1 Master Transmit Output
   */
  P10_IOCR4.B.PC5 = PCx_Output_PushPull_Alt_4;
  P11_IOCR4.B.PC6 = PCx_Output_PushPull_Alt_3;
  P11_IOCR0.B.PC3 = PCx_Input_PullDown;
  P11_IOCR8.B.PC9 = PCx_Output_PushPull_Alt_3;

  /* Set the driver strength*/
  IfxScuWdt_clearCpuEndinit(cpuWdtPassword);
  P11_PDR0.U = PDx_Speed_Grade_1;
  P11_PDR1.U = PDx_Speed_Grade_1;
  IfxScuWdt_setCpuEndinit(cpuWdtPassword);

  /* Enable SPI with user configuration */
  QSPI1_GLOBALCON.B.EN = qspi_globalcon_en_RUN;
}


/*! \fn
 *  \brief
 *  \param
 *  \return
 */
void QSPI1_SendPacketU8(uint8 *src, uint8 *dst, uint8 txCnt)
{
  uint32 cntSend;
  uint32 cntRead;

  /*there was an error in the slave spi exchange */
  QSPI1_FLAGSCLEAR.U = 0x9FFF;

  /*Load QSPI BACON word */
  QSPI1_BACONENTRY.U = baconEntrySpi1Start.U;

  cntRead = 0;
  cntSend = txCnt - 1;
  while(cntSend)
  {
    if (QSPI1_STATUS.B.TXFIFOLEVEL < 4)
    {
      QSPI1_DATAENTRY0.U = *src++;
      cntSend--;
    }
    /* unload the received SPI data*/
    if (QSPI1_STATUS.B.RXFIFOLEVEL > 0)
    {
      *dst++ = (uint8)QSPI1_RXEXIT.U;
      cntRead++;
    }
  }

  /*close the frame */
  while (QSPI1_STATUS.B.TXFIFOLEVEL > 1);
  QSPI1_BACONENTRY.U = baconEntrySpi1Stop.U;
  /*write the last data word to the QSPI */
  QSPI1_DATAENTRY0.U = *src;
  /* unload the received SPI data*/
  while(cntRead != txCnt)
  {
    if (QSPI1_STATUS.B.RXFIFOLEVEL > 0)
    {
      *dst++ = (uint8)QSPI1_RXEXIT.U;
      cntRead++;
    }
  }
}

/*! \fn
 *  \brief
 *  \param
 *  \return
 */
void QSPI1_SendPacketU16(uint16 *src, uint16 *dst, uint8 txCnt)
{
  uint32 cntSend;
  uint32 cntRead;

  /*there was an error in the slave spi exchange */
  QSPI1_FLAGSCLEAR.U = 0x9FFF;

  /*Load QSPI BACON word */
  QSPI1_BACONENTRY.U = baconEntrySpi1Start.U;

  cntRead = 0;
  cntSend = txCnt - 1;
  while(cntSend)
  {
    if (QSPI1_STATUS.B.TXFIFOLEVEL < 4)
    {
      QSPI1_DATAENTRY0.U = *src++;
      cntSend--;
    }
    /* unload the received SPI data*/
    if (QSPI1_STATUS.B.RXFIFOLEVEL > 0)
    {
      *dst++ = (uint16)QSPI1_RXEXIT.U;
      cntRead++;
    }
  }

  /*close the frame */
  while (QSPI1_STATUS.B.TXFIFOLEVEL > 1);
  QSPI1_BACONENTRY.U = baconEntrySpi1Stop.U;
  /*write the last data word to the QSPI */
  QSPI1_DATAENTRY0.U = *src;
  /* unload the received SPI data*/
  while(cntRead != txCnt)
  {
    if (QSPI1_STATUS.B.RXFIFOLEVEL > 0)
    {
      *dst++ = (uint16)QSPI1_RXEXIT.U;
      cntRead++;
    }
  }
}

/*! \fn void QSPI1_SendPacket(uint32 *src, uint32 *dst, uint8 txCnt)
 *  \brief Sends one packet of QSPI data (blocking function)
 *  \param value: data that is up to 32-bits long to send on the QSPI
 *  \return nothing
 */
void QSPI1_SendPacketU32(uint32 *src, uint32 *dst, uint8 txCnt)
{
  uint32 cntSend;
  uint32 cntRead;

  /*there was an error in the slave spi exchange */
  QSPI1_FLAGSCLEAR.U = 0x9FFF;

  /*Load QSPI BACON word */
  QSPI1_BACONENTRY.U = baconEntrySpi1Start.U;

  cntRead = 0;
  cntSend = txCnt - 1;
  while(cntSend)
  {
    if (QSPI1_STATUS.B.TXFIFOLEVEL < 4)
    {
      QSPI1_DATAENTRY0.U = *src++;
      cntSend--;
    }
    /* unload the received SPI data*/
    if (QSPI1_STATUS.B.RXFIFOLEVEL > 0)
    {
      *dst++ = QSPI1_RXEXIT.U;
      cntRead++;
    }
  }

  /*close the frame */
  while (QSPI1_STATUS.B.TXFIFOLEVEL > 1);
  QSPI1_BACONENTRY.U = baconEntrySpi1Stop.U;
  /*write the last data word to the QSPI */
  QSPI1_DATAENTRY0.U = *src;
  /* unload the received SPI data*/
  while(cntRead != txCnt)
  {
    if (QSPI1_STATUS.B.RXFIFOLEVEL > 0)
    {
      *dst++ = QSPI1_RXEXIT.U;
      cntRead++;
    }
  }
}

/*! \fn uint32 QSPI1_ExchangeData(uint32_t value)
 *  \brief If you want to poll the SPI transfers and the RXFIFO is empty
 *  \param value: data that is up to 32-bits long to send on the QSPI
 *  \return The received data from the QSPI (up to 32-bits of data)
 */
uint32 QSPI1_ExchangeData(uint32 value)
{
  QSPI1_DATAENTRY0.U = value;
  while (QSPI1_STATUS.B.RXFIFOLEVEL == 0);
  QSPI1_FLAGSCLEAR.U = 0xFFFF;
  return QSPI1_RXEXIT.U;
}


/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
IFX_INTERRUPT(QSPI1_U_ISR, VECTAB0, SRPN_CPU0_QSPI1_U);
void QSPI1_U_ISR(void)
{

}

/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
IFX_INTERRUPT(QSPI1_PT_ISR, VECTAB0, SRPN_CPU0_QSPI1_PT);
void QSPI1_PT_ISR(void)
{

}

/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
IFX_INTERRUPT(QSPI1_Err_ISR, VECTAB0, SRPN_CPU0_QSPI1_ERR);
void QSPI1_Err_ISR(void)
{
  QSPI1_errorCount++;
}

/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
IFX_INTERRUPT(QSPI1_Rx_ISR, VECTAB0, SRPN_CPU0_QSPI1_RX);
void QSPI1_Rx_ISR(void)
{

}


/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
IFX_INTERRUPT(QSPI1_Tx_ISR, VECTAB0, SRPN_CPU0_QSPI1_TX);
void QSPI1_Tx_ISR(void)
{

}


/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
void QSPI1_StoreBACON(Ifx_QSPI_BACON bacon)
{
	QSPI1_BACONENTRY.U = bacon.U;
}


/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
void QSPI1_ConfigServiceInterrupts(void)
{
  /* Enter interrupt level and initialize both the SRC and table entry */
  SRC_QSPI1RX.U  = TOS_CPU0 | SRE_ON  | SRPN_CPU0_QSPI1_RX;
  SRC_QSPI1TX.U  = TOS_CPU0 | SRE_ON  | SRPN_CPU0_QSPI1_TX;
}

/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
void QSPI1_ConfigServicePolled(void)
{
  /* Enter interrupt level and initialize both the SRC and table entry */
  SRC_QSPI1RX.U  = TOS_CPU0 | SRE_OFF | SRPN_CPU0_QSPI1_RX;
  SRC_QSPI1TX.U  = TOS_CPU0 | SRE_OFF | SRPN_CPU0_QSPI1_TX;
}

/*! \fn
 *  \brief
 *  \param None
 *  \return Nothing
 */
void QSPI1_ConfigServiceDMA(void)
{
 /* use DMA */
  SRC_QSPI1TX.U  = TOS_DMA  | SRE_ON  | SRPN_DMA_CH_QSPI1_TX;
  SRC_QSPI1RX.U  = TOS_DMA  | SRE_ON  | SRPN_DMA_CH_QSPI1_RX | CLRR;
}

/** \brief
 *
 * \param None
 * \return Nothing
 *
 * \ingroup
 */
IFX_INTERRUPT(DMA_Ch3ISR, VECTAB0, SRPN_CPU0_DMA_CH3);
void DMA_Ch3ISR(void)
{
#if QSPI_DL==DMA_CH_SIZE_INT8
uint8 *src = (uint8 *)DMA_SADR003.U;
#elif QSPI_DL==DMA_CH_SIZE_INT16
uint16 *src = (uint16 *)DMA_SADR003.U;
#else
uint32 *src = (uint32 *)DMA_SADR003.U;
#endif

  /* transmit of all but last data entry has occurred,
   * but we need to close the transfer by changing the LAST=DONE*/

  /*close the frame */

  /* wait for two entries to be free in the TXFIFO*/
  while (QSPI1_STATUS.B.TXFIFOLEVEL > 1);
  /* write the BACON with LAST=1*/
  QSPI1_BACONENTRY.U = baconEntrySpi1Stop.U;
  /*write the last data entry to the QSPI */
  QSPI1_DATAENTRY0.U = *src;

  /*Transmit interrupt flag handler
   * used for debug to know the transfer completed*/
  dmaCh3Cnt++;
}

/** \brief
 *
 * \param None
 * \return Nothing
 *
 * \ingroup
 */
IFX_INTERRUPT(DMA_Ch4ISR, VECTAB0, SRPN_CPU0_DMA_CH4);
void DMA_Ch4ISR(void)
{
  /* All data received flag handler
   * used for debug to know the transfer completed*/
  dmaCh4Cnt++;
}


/** \brief
 *
 * \param None
 * \return Nothing
 *
 * \ingroup
 */
void DMA_QSPI1_Init(uint16 trel, uint8 channelWidth, uint32 src, uint32 dst)
{
  /* DMA Channel 003 used for QSPI Transmit */
  DMA_SADR003.U = src;
  DMA_DADR003.U = (uint32)&QSPI1_DATAENTRY0.U;

  Ifx_DMA_CH_CHCFGR chcfgr_3 =
  {
    .B.TREL = trel-1,    /*Transfer Reload Value */
    .B.BLKM = dma_chcfgrxxx_blkm_OneTransferHasOneMove,
    .B.RROAT = dma_chcfgrxxx_rroat_ResetAfterEachTransfer,
    .B.CHMODE = dma_chcfgrxxx_chmode_SingleMode,
    .B.CHDW = channelWidth,
    .B.PATSEL = dma_chcfgrxxx_patsel_NoPatternCompareOperation,
    .B.SWAP = dma_chcfgrxxx_swap_ByteOrderIsNotSwapped,
    .B.PRSEL = dma_chcfgrxxx_prsel_HardwareRequestSelected,
  };
  DMA_CHCFGR003.U = chcfgr_3.U;

  Ifx_DMA_CH_ADICR adicr_3 =
  {
    .B.SMF = dma_adicrxxx_smf_AddressOffsetIsCHDWx1,
    .B.INCS = dma_adicrxxx_incs_AddressOffsetIsAdded,
    .B.DMF = dma_adicrxxx_dmf_AddressOffsetIsCHDWx1,
    .B.INCD = dma_adicrxxx_incd_AddressOffsetIsSubtracted,
    .B.CBLS = dma_adicrxxx_cbls_SourceAddress31to0,
    .B.CBLD = dma_adicrxxx_cbld_DestinationAddress31to0,
    .B.SHCT = dma_adicrxxx_shct_MoveOperation,
    .B.SCBE = dma_adicrxxx_scbe_SourceCircularBufferDisabled,
    .B.DCBE = dma_adicrxxx_dcbe_DestinationCircularBufferEnabled,
    .B.STAMP = dma_adicrxxx_stamp_NoAction,
    .B.WRPSE = dma_adicrxxx_wrpse_WrapSourceBufferInterruptTriggerDiabled,
    .B.WRPDE = dma_adicrxxx_wrpde_WrapDestinationBufferInterruptTriggerDiabled,
    .B.INTCT = dma_adicrxxx_intct_InterruptChangingTCOUNTandEqualsIRDV,
    .B.IRDV = 0,  /*Interrupt Raise Detect Value*/
  };
  DMA_ADICR003.U = adicr_3.U;
  DMA_TSR003.B.ECH = 1;

  /* DMA Channel 004: used for QSPI Receive */
  DMA_SADR004.U = (uint32)&QSPI1_RXEXIT.U;
  DMA_DADR004.U = dst;

  Ifx_DMA_CH_CHCFGR chcfgr_4 =
  {
    .B.TREL = trel, /*Transfer Reload Value */
    .B.BLKM = dma_chcfgrxxx_blkm_OneTransferHasOneMove,
    .B.RROAT = dma_chcfgrxxx_rroat_ResetAfterEachTransfer,
    .B.CHMODE = dma_chcfgrxxx_chmode_SingleMode,
    .B.CHDW = channelWidth,
    .B.PATSEL = dma_chcfgrxxx_patsel_NoPatternCompareOperation,
    .B.SWAP = dma_chcfgrxxx_swap_ByteOrderIsNotSwapped,
    .B.PRSEL = dma_chcfgrxxx_prsel_HardwareRequestSelected,
  };
  DMA_CHCFGR004.U = chcfgr_4.U;

  Ifx_DMA_CH_ADICR adicr_4 =
  {
    .B.SMF = dma_adicrxxx_smf_AddressOffsetIsCHDWx1,
    .B.INCS = dma_adicrxxx_incs_AddressOffsetIsSubtracted,
    .B.DMF = dma_adicrxxx_dmf_AddressOffsetIsCHDWx1,
    .B.INCD = dma_adicrxxx_incd_AddressOffsetIsAdded,
    .B.CBLS = dma_adicrxxx_cbls_SourceAddress31to0,
    .B.CBLD = dma_adicrxxx_cbld_DestinationAddress31to0,
    .B.SHCT = dma_adicrxxx_shct_MoveOperation,
    .B.SCBE = dma_adicrxxx_scbe_SourceCircularBufferEnabled,
    .B.DCBE = dma_adicrxxx_dcbe_DestinationCircularBufferDisabled,
    .B.STAMP = dma_adicrxxx_stamp_NoAction,
    .B.WRPSE = dma_adicrxxx_wrpse_WrapSourceBufferInterruptTriggerDiabled,
    .B.WRPDE = dma_adicrxxx_wrpde_WrapDestinationBufferInterruptTriggerDiabled,
    .B.INTCT =dma_adicrxxx_intct_InterruptChangingTCOUNTandEqualsIRDV,
    .B.IRDV = 0,  /*Interrupt Raise Detect Value*/
  };
  DMA_ADICR004.U = adicr_4.U;
  DMA_TSR004.B.ECH = 1;

  /* Setup interrupt from DMA Channel 5 and 6 when the Move Transaction has finished */
  SRC_DMACH3.U = TOS_CPU0 | SRE_ON | SRPN_CPU0_DMA_CH3;
  SRC_DMACH4.U = TOS_CPU0 | SRE_ON | SRPN_CPU0_DMA_CH4;
}


/*************************************************************************
 Development history of the file



*************************************************************************/
